package com.sun.jmx.mbeanserver;

import javax.management.MBeanServerPermission;
import javax.management.MBeanPermission;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import java.security.PrivilegedExceptionAction;
import javax.management.RuntimeOperationsException;
import javax.management.OperationsException;
import java.io.ObjectInputStream;
import javax.management.IntrospectionException;
import javax.management.MBeanInfo;
import javax.management.ListenerNotFoundException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.InvalidAttributeValueException;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import java.util.Set;
import javax.management.QueryExp;
import javax.management.InstanceNotFoundException;
import javax.management.NotCompliantMBeanException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.ReflectionException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import com.sun.jmx.interceptor.DefaultMBeanServerInterceptor;
import java.security.AccessController;
import java.security.PrivilegedAction;
import javax.management.loading.ClassLoaderRepository;
import javax.management.MBeanServerDelegate;
import javax.management.MBeanServer;

public final class JmxMBeanServer implements SunJmxMBeanServer
{
    public static final boolean DEFAULT_FAIR_LOCK_POLICY = true;
    private final MBeanInstantiator instantiator;
    private final SecureClassLoaderRepository secureClr;
    private final boolean interceptorsEnabled;
    private final MBeanServer outerShell;
    private volatile MBeanServer mbsInterceptor;
    private final MBeanServerDelegate mBeanServerDelegateObject;
    
    JmxMBeanServer(String s, MBeanServer mBeanServer, MBeanServerDelegate mBeanServerDelegate) {
        this(s, mBeanServer, mBeanServerDelegate, null, false);
    }
    
    JmxMBeanServer(String s, MBeanServer mBeanServer, MBeanServerDelegate mBeanServerDelegate, boolean b) {
        this(s, mBeanServer, mBeanServerDelegate, null, false);
    }
    
    JmxMBeanServer(String s, MBeanServer mBeanServer, MBeanServerDelegate mBeanServerDelegate, MBeanInstantiator mBeanInstantiator, boolean b) {
        this(s, mBeanServer, mBeanServerDelegate, mBeanInstantiator, b, true);
    }
    
    JmxMBeanServer(String s, final MBeanServer mBeanServer, MBeanServerDelegate mBeanServerDelegate, final MBeanInstantiator mBeanInstantiator, boolean interceptorsEnabled, boolean b) {
        new Object();
        mBeanServer.mbsInterceptor = null;
        if (mBeanInstantiator == null) {
            mBeanInstantiator = new MBeanInstantiator(new ClassLoaderRepositorySupport());
        }
        mBeanServer.secureClr = new SecureClassLoaderRepository((ClassLoaderRepository)AccessController.doPrivileged(new PrivilegedAction<ClassLoaderRepository>() {
            @Override
            public ClassLoaderRepository run() {
                return mBeanInstantiator.getClassLoaderRepository();
            }
        }));
        if (mBeanServerDelegate == null) {
            mBeanServerDelegate = new MBeanServerDelegateImpl();
        }
        if (mBeanServer == null) {
        }
        mBeanServer.instantiator = mBeanInstantiator;
        mBeanServer.mBeanServerDelegateObject = mBeanServerDelegate;
        mBeanServer.outerShell = mBeanServer;
        mBeanServer.mbsInterceptor = new DefaultMBeanServerInterceptor(mBeanServer, mBeanServerDelegate, mBeanInstantiator, new Repository(s));
        mBeanServer.interceptorsEnabled = interceptorsEnabled;
        mBeanServer.initialize();
    }
    
    @Override
    public boolean interceptorsEnabled() {
        return this.interceptorsEnabled;
    }
    
    @Override
    public MBeanInstantiator getMBeanInstantiator() {
        if (this.interceptorsEnabled) {
            return this.instantiator;
        }
        throw new UnsupportedOperationException("MBeanServerInterceptors are disabled.");
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException {
        return this.mbsInterceptor.createMBean(s, this.cloneObjectName(objectName), (Object[])null, (String[])null);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, ObjectName objectName2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException {
        return this.mbsInterceptor.createMBean(s, this.cloneObjectName(objectName), objectName2, (Object[])null, (String[])null);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, Object[] array, String[] array2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException {
        return this.mbsInterceptor.createMBean(s, this.cloneObjectName(objectName), array, array2);
    }
    
    @Override
    public ObjectInstance createMBean(String s, ObjectName objectName, ObjectName objectName2, Object[] array, String[] array2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException {
        return this.mbsInterceptor.createMBean(s, this.cloneObjectName(objectName), objectName2, array, array2);
    }
    
    @Override
    public ObjectInstance registerMBean(Object o, ObjectName objectName) throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
        return this.mbsInterceptor.registerMBean(o, this.cloneObjectName(objectName));
    }
    
    @Override
    public void unregisterMBean(ObjectName objectName) throws InstanceNotFoundException, MBeanRegistrationException {
        this.mbsInterceptor.unregisterMBean(this.cloneObjectName(objectName));
    }
    
    @Override
    public ObjectInstance getObjectInstance(ObjectName objectName) throws InstanceNotFoundException {
        return this.mbsInterceptor.getObjectInstance(this.cloneObjectName(objectName));
    }
    
    @Override
    public Set<ObjectInstance> queryMBeans(ObjectName objectName, QueryExp queryExp) {
        return this.mbsInterceptor.queryMBeans(this.cloneObjectName(objectName), queryExp);
    }
    
    @Override
    public Set<ObjectName> queryNames(ObjectName objectName, QueryExp queryExp) {
        return this.mbsInterceptor.queryNames(this.cloneObjectName(objectName), queryExp);
    }
    
    @Override
    public boolean isRegistered(ObjectName objectName) {
        return this.mbsInterceptor.isRegistered(objectName);
    }
    
    @Override
    public Integer getMBeanCount() {
        return this.mbsInterceptor.getMBeanCount();
    }
    
    @Override
    public Object getAttribute(ObjectName objectName, String s) throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException {
        return this.mbsInterceptor.getAttribute(this.cloneObjectName(objectName), s);
    }
    
    @Override
    public AttributeList getAttributes(ObjectName objectName, String[] array) throws InstanceNotFoundException, ReflectionException {
        return this.mbsInterceptor.getAttributes(this.cloneObjectName(objectName), array);
    }
    
    @Override
    public void setAttribute(ObjectName objectName, Attribute attribute) throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        this.mbsInterceptor.setAttribute(this.cloneObjectName(objectName), this.cloneAttribute(attribute));
    }
    
    @Override
    public AttributeList setAttributes(ObjectName objectName, AttributeList list) throws InstanceNotFoundException, ReflectionException {
        return this.mbsInterceptor.setAttributes(this.cloneObjectName(objectName), this.cloneAttributeList(list));
    }
    
    @Override
    public Object invoke(ObjectName objectName, String s, Object[] array, String[] array2) throws InstanceNotFoundException, MBeanException, ReflectionException {
        return this.mbsInterceptor.invoke(this.cloneObjectName(objectName), s, array, array2);
    }
    
    @Override
    public String getDefaultDomain() {
        return this.mbsInterceptor.getDefaultDomain();
    }
    
    @Override
    public String[] getDomains() {
        return this.mbsInterceptor.getDomains();
    }
    
    @Override
    public void addNotificationListener(ObjectName objectName, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException {
        this.mbsInterceptor.addNotificationListener(this.cloneObjectName(objectName), notificationListener, notificationFilter, o);
    }
    
    @Override
    public void addNotificationListener(ObjectName objectName, ObjectName objectName2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException {
        this.mbsInterceptor.addNotificationListener(this.cloneObjectName(objectName), objectName2, notificationFilter, o);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener) throws InstanceNotFoundException, ListenerNotFoundException {
        this.mbsInterceptor.removeNotificationListener(this.cloneObjectName(objectName), notificationListener);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException {
        this.mbsInterceptor.removeNotificationListener(this.cloneObjectName(objectName), notificationListener, notificationFilter, o);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, ObjectName objectName2) throws InstanceNotFoundException, ListenerNotFoundException {
        this.mbsInterceptor.removeNotificationListener(this.cloneObjectName(objectName), objectName2);
    }
    
    @Override
    public void removeNotificationListener(ObjectName objectName, ObjectName objectName2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException {
        this.mbsInterceptor.removeNotificationListener(this.cloneObjectName(objectName), objectName2, notificationFilter, o);
    }
    
    @Override
    public MBeanInfo getMBeanInfo(ObjectName objectName) throws InstanceNotFoundException, IntrospectionException, ReflectionException {
        return this.mbsInterceptor.getMBeanInfo(this.cloneObjectName(objectName));
    }
    
    @Override
    public Object instantiate(String s) throws ReflectionException, MBeanException {
        checkMBeanPermission(s, null, null, "instantiate");
        return this.instantiator.instantiate(s);
    }
    
    @Override
    public Object instantiate(String s, ObjectName objectName) throws ReflectionException, MBeanException, InstanceNotFoundException {
        checkMBeanPermission(s, null, null, "instantiate");
        return this.instantiator.instantiate(s, objectName, this.outerShell.getClass().getClassLoader());
    }
    
    @Override
    public Object instantiate(String s, Object[] array, String[] array2) throws ReflectionException, MBeanException {
        checkMBeanPermission(s, null, null, "instantiate");
        return this.instantiator.instantiate(s, array, array2, this.outerShell.getClass().getClassLoader());
    }
    
    @Override
    public Object instantiate(String s, ObjectName objectName, Object[] array, String[] array2) throws ReflectionException, MBeanException, InstanceNotFoundException {
        checkMBeanPermission(s, null, null, "instantiate");
        return this.instantiator.instantiate(s, objectName, array, array2, this.outerShell.getClass().getClassLoader());
    }
    
    @Override
    public boolean isInstanceOf(ObjectName objectName, String s) throws InstanceNotFoundException {
        return this.mbsInterceptor.isInstanceOf(this.cloneObjectName(objectName), s);
    }
    
    @Deprecated
    @Override
    public ObjectInputStream deserialize(ObjectName objectName, byte[] array) throws InstanceNotFoundException, OperationsException {
        return this.instantiator.deserialize(this.getClassLoaderFor(objectName), array);
    }
    
    @Deprecated
    @Override
    public ObjectInputStream deserialize(String s, byte[] array) throws OperationsException, ReflectionException {
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException(), "Null className passed in parameter");
        }
        ClassLoaderRepository classLoaderRepository = this.getClassLoaderRepository();
        Class<Object> loadClass;
        try {
            if (classLoaderRepository == null) {
                throw new ClassNotFoundException(s);
            }
            loadClass = classLoaderRepository.loadClass(s);
        }
        catch (ClassNotFoundException e) {
            throw new ReflectionException(e, "The given class could not be loaded by the default loader repository");
        }
        return this.instantiator.deserialize(loadClass.getClassLoader(), array);
    }
    
    @Deprecated
    @Override
    public ObjectInputStream deserialize(String s, ObjectName cloneObjectName, byte[] array) throws InstanceNotFoundException, OperationsException, ReflectionException {
        cloneObjectName = this.cloneObjectName(cloneObjectName);
        try {
            this.getClassLoader(cloneObjectName);
        }
        catch (SecurityException ex) {
            throw ex;
        }
        catch (Exception ex2) {}
        return this.instantiator.deserialize(s, cloneObjectName, array, this.outerShell.getClass().getClassLoader());
    }
    
    private void initialize() {
        if (this.instantiator == null) {
            throw new IllegalStateException("instantiator must not be null.");
        }
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
                @Override
                public Object run() throws Exception {
                    JmxMBeanServer.this.mbsInterceptor.registerMBean(JmxMBeanServer.this.mBeanServerDelegateObject, MBeanServerDelegate.DELEGATE_NAME);
                    return null;
                }
            });
        }
        catch (SecurityException thrown) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, JmxMBeanServer.class.getName(), "initialize", "Unexpected security exception occurred", thrown);
            }
            throw thrown;
        }
        catch (Exception ex) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINEST, JmxMBeanServer.class.getName(), "initialize", "Unexpected exception occurred", ex);
            }
            throw new IllegalStateException("Can't register delegate.", ex);
        }
        ClassLoader classLoader = this.outerShell.getClass().getClassLoader();
        ModifiableClassLoaderRepository modifiableClassLoaderRepository = (ModifiableClassLoaderRepository)AccessController.doPrivileged(new PrivilegedAction<ModifiableClassLoaderRepository>() {
            @Override
            public ModifiableClassLoaderRepository run() {
                return JmxMBeanServer.this.instantiator.getClassLoaderRepository();
            }
        });
        if (modifiableClassLoaderRepository != null) {
            modifiableClassLoaderRepository.addClassLoader(classLoader);
            ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
            if (systemClassLoader != classLoader) {
                modifiableClassLoaderRepository.addClassLoader(systemClassLoader);
            }
        }
        return;
    }
    
    @Override
    public synchronized MBeanServer getMBeanServerInterceptor() {
        if (this.interceptorsEnabled) {
            return this.mbsInterceptor;
        }
        throw new UnsupportedOperationException("MBeanServerInterceptors are disabled.");
    }
    
    @Override
    public synchronized void setMBeanServerInterceptor(MBeanServer mbsInterceptor) {
        if (!this.interceptorsEnabled) {
            throw new UnsupportedOperationException("MBeanServerInterceptors are disabled.");
        }
        if (mbsInterceptor == null) {
            throw new IllegalArgumentException("MBeanServerInterceptor is null");
        }
        this.mbsInterceptor = mbsInterceptor;
        return;
    }
    
    @Override
    public ClassLoader getClassLoaderFor(ObjectName objectName) throws InstanceNotFoundException {
        return this.mbsInterceptor.getClassLoaderFor(this.cloneObjectName(objectName));
    }
    
    @Override
    public ClassLoader getClassLoader(ObjectName objectName) throws InstanceNotFoundException {
        return this.mbsInterceptor.getClassLoader(this.cloneObjectName(objectName));
    }
    
    @Override
    public ClassLoaderRepository getClassLoaderRepository() {
        checkMBeanPermission(null, null, null, "getClassLoaderRepository");
        return this.secureClr;
    }
    
    @Override
    public MBeanServerDelegate getMBeanServerDelegate() {
        if (!this.interceptorsEnabled) {
            throw new UnsupportedOperationException("MBeanServerInterceptors are disabled.");
        }
        return this.mBeanServerDelegateObject;
    }
    
    public static MBeanServerDelegate newMBeanServerDelegate() {
        return new MBeanServerDelegateImpl();
    }
    
    public static MBeanServer newMBeanServer(String s, MBeanServer mBeanServer, MBeanServerDelegate mBeanServerDelegate, boolean b) {
        checkNewMBeanServerPermission();
        return new JmxMBeanServer(s, mBeanServer, mBeanServerDelegate, null, b, true);
    }
    
    private ObjectName cloneObjectName(ObjectName name) {
        if (name != null) {
            return ObjectName.getInstance(name);
        }
        return name;
    }
    
    private Attribute cloneAttribute(Attribute attribute) {
        if (attribute != null && !attribute.getClass().equals(Attribute.class)) {
            return new Attribute(attribute.getName(), attribute.getValue());
        }
        return attribute;
    }
    
    private AttributeList cloneAttributeList(AttributeList list) {
        if (list != null) {
            List<Attribute> list2 = list.asList();
            if (!list.getClass().equals(AttributeList.class)) {
                AttributeList list3 = new AttributeList(list2.size());
                Iterator<Object> iterator = list2.iterator();
                while (iterator.hasNext()) {
                    list3.add(this.cloneAttribute((Attribute)iterator.next()));
                }
                return list3;
            }
            for (int i = 0; i < list2.size(); ++i) {
                Attribute attribute = (Attribute)list2.get(i);
                if (!attribute.getClass().equals(Attribute.class)) {
                    list.set(i, this.cloneAttribute(attribute));
                }
            }
            return list;
        }
        else {
            return list;
        }
    }
    
    private static void checkMBeanPermission(String className, String member, ObjectName objectName, String actions) throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new MBeanPermission(className, member, objectName, actions));
        }
    }
    
    private static void checkNewMBeanServerPermission() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new MBeanServerPermission("newMBeanServer"));
        }
    }
}
